@@ -3,9 +3,11 @@ from abc import ABC, abstractmethod
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 import pandas  # type: ignore
+from cached_property import cached_property  # type: ignore
 
 from tktl.core.exceptions import exceptions
 from tktl.core.t import EndpointKinds
+from tktl.core.utils import DelayedLoadingFrame, check_and_get_value
 from tktl.registration.schema import EndpointInputSchema, EndpointOutputSchema
 from tktl.registration.validation import (
     input_pandas_representation,
@@ -21,6 +23,7 @@ class Endpoint(ABC):
     input_schema: EndpointInputSchema
     output_schema: EndpointOutputSchema
     explain_input_schema: Optional[EndpointInputSchema]
+    delayed_loading: bool
 
     def __init__(
         self,
@@ -29,19 +32,20 @@ class Endpoint(ABC):
         disable_rest_validation: bool = True,
     ):
         self._func = func
+        self.delayed_loading = False
         self.profile_columns = (
             [str(c) for c in profile_columns] if profile_columns is not None else None
         )
         self.disable_rest_validation = disable_rest_validation
 
-    @property
+    @cached_property
     def has_arrow_sample_data(self):
         return (
             self.input_schema.supports_arrow_sample_data
             and self.output_schema.supports_arrow_sample_data
         )
 
-    @property
+    @cached_property
     def has_rest_sample_data(self):
         if self.disable_rest_validation:
             return False
@@ -65,16 +69,16 @@ class Endpoint(ABC):
         to_series.__name__ = name
         return to_series
 
-    @property
+    @cached_property
     @abstractmethod
     def profiling_supported(self):
         return True
 
-    @property
+    @cached_property
     def input_names(self):
         return []
 
-    @property
+    @cached_property
     def output_names(self):
         return []
 
@@ -87,20 +91,45 @@ class TabularEndpoint(Endpoint):
         super().__init__(
             func, profile_columns, disable_rest_validation=disable_rest_validation
         )
+
+        if isinstance(X, str) and isinstance(y, str):
+            self.delayed_loading = True
+
+            x_value = DelayedLoadingFrame(X, y, profile_columns=profile_columns)
+            y_value = DelayedLoadingFrame(
+                X, y, is_label=True, profile_columns=profile_columns,
+            )
+
+        else:
+            x_value = X
+            y_value = y
+
         self.input_schema = EndpointInputSchema(
-            value=X, endpoint_kind=self.KIND, endpoint_name=self.func.__name__
+            value=x_value, endpoint_kind=self.KIND, endpoint_name=self.func.__name__
+        )
+        self.output_schema = EndpointOutputSchema(
+            value=y_value, endpoint_kind=self.KIND, endpoint_name=self.func.__name__
         )
 
-        self.output_schema = EndpointOutputSchema(
-            value=y, endpoint_kind=self.KIND, endpoint_name=self.func.__name__
-        )
         if self.profiling_supported:
-            input_value = input_pandas_representation(self.input_schema.value)
             if not profile_columns:
-                self.profile_columns = [v for v in input_value.columns]
+                self.profile_columns = self.input_schema.names
+
+            if self.delayed_loading:
+                explain_input_value = DelayedLoadingFrame(
+                    X,
+                    y,
+                    is_label=False,
+                    profile_columns=self.profile_columns,
+                    load_for_profiling=True,
+                )
+            else:
+                explain_input_value = self.input_schema.pandas_representation[
+                    self.profile_columns
+                ]
 
             self.explain_input_schema = EndpointInputSchema(
-                value=input_value[self.profile_columns],
+                value=explain_input_value,
                 endpoint_kind=self.KIND,
                 endpoint_name=f"explain__{self.func.__name__}",
             )
@@ -115,14 +144,15 @@ class TabularEndpoint(Endpoint):
 
             return False
         try:
-            predictions = self.func(self.input_schema.value)
+            value = check_and_get_value(self.input_schema.value)
+            predictions = self.func(value)
         except Exception as e:
             raise exceptions.ValidationException(
                 f"Function provided is unable to produce predictions from given sample values: {e}"
             )
         return predictions
 
-    @property
+    @cached_property
     def profiling_supported(self):
         predictions = self._profiling_supported()
         if predictions is False:
@@ -137,15 +167,15 @@ class TabularEndpoint(Endpoint):
                 x_frame, y_series, predictions
             )
 
-    @property
+    @cached_property
     def input_names(self):
         return self.input_schema.names
 
-    @property
+    @cached_property
     def explain_input_names(self):
         return self.explain_input_schema.names if self.explain_input_schema else []
 
-    @property
+    @cached_property
     def output_names(self):
         return self.output_schema.names if self.explain_input_schema else []
 
@@ -158,7 +188,7 @@ class BinaryEndpoint(TabularEndpoint):
             func, X, y, profile_columns, disable_rest_validation=disable_rest_validation
         )
 
-    @property
+    @cached_property
     def profiling_supported(self):
         predictions = self._profiling_supported()
         if predictions is False:
@@ -174,7 +204,7 @@ class RegressionEndpoint(TabularEndpoint):
             func, X, y, profile_columns, disable_rest_validation=disable_rest_validation
         )
 
-    @property
+    @cached_property
     def profiling_supported(self):
         predictions = self._profiling_supported()
         if predictions is False:
@@ -230,7 +260,7 @@ class CustomEndpoint(Endpoint):
             user_defined_model=response_model,
         )
 
-    @property
+    @cached_property
     def profiling_supported(self):
         return False
 
