@@ -6,6 +6,7 @@ import pandas.api.types as ptypes  # type: ignore
 
 from tktl.core.exceptions import exceptions
 from tktl.core.loggers import LOG
+from tktl.core.utils import check_and_get_value
 
 
 def validate_outputs(preds) -> Union[bool, Iterator]:
@@ -93,7 +94,7 @@ def input_pandas_representation(
     -------
     df : pandas.DataFrame
     """
-
+    value = check_and_get_value(value)
     df = pandas.DataFrame(value)
     df.columns = [str(c) for c in df.columns]
     if reset_index:
@@ -120,7 +121,7 @@ def output_pandas_representation(
     -------
     series : pandas.Series
     """
-
+    value = check_and_get_value(value)
     if isinstance(value, pandas.Series):
         if reset_index:
             value = value.reset_index(drop=True)
